@@ -41,15 +41,17 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         self._configure = configure
         self._reset = reset
 
-        self._lock = asyncio.Lock()
+        # asyncio objects, created on open to attach them to the right loop.
+        self._lock: asyncio.Lock
+        self._sched: AsyncScheduler
+        self._tasks: "asyncio.Queue[MaintenanceTask]"
+
         self._waiting = Deque["AsyncClient"]()
 
         # to notify that the pool is full
         self._pool_full_event: Optional[asyncio.Event] = None
 
-        self._sched = AsyncScheduler()
         self._sched_runner: Optional[Task[None]] = None
-        self._tasks: "asyncio.Queue[MaintenanceTask]" = asyncio.Queue()
         self._workers: List[Task[None]] = []
 
         super().__init__(conninfo, **kwargs)
@@ -62,7 +64,7 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
 
         async with self._lock:
             assert not self._pool_full_event
-            if len(self._pool) >= self._nconns:
+            if len(self._pool) >= self._min_size:
                 return
             self._pool_full_event = asyncio.Event()
 
@@ -99,10 +101,11 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         logger.info("connection requested from %r", self.name)
         self._stats[self._REQUESTS_NUM] += 1
 
+        self._check_open_getconn()
+
         # Critical section: decide here if there's a connection ready
         # or if the client needs to wait.
         async with self._lock:
-            self._check_open_getconn()
             conn = await self._get_ready_connection(timeout)
             if not conn:
                 # No connection available: put the client in the waiting queue
@@ -185,6 +188,12 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         return True
 
     async def open(self, wait: bool = False, timeout: float = 30.0) -> None:
+        # Make sure the lock is created after there is an event loop
+        try:
+            self._lock
+        except AttributeError:
+            self._lock = asyncio.Lock()
+
         async with self._lock:
             self._open()
 
@@ -195,8 +204,21 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         if not self._closed:
             return
 
+        # Throw a RuntimeError if the pool is open outside a running loop.
+        asyncio.get_running_loop()
+
         self._check_open()
 
+        # Create these objects now to attach them to the right loop.
+        # See #219
+        self._tasks = asyncio.Queue()
+        self._sched = AsyncScheduler()
+        # This has been most likely, but not necessarily, created in `open()`.
+        try:
+            self._lock
+        except AttributeError:
+            self._lock = asyncio.Lock()
+
         self._closed = False
         self._opened = True
 
@@ -428,6 +450,9 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
                 )
                 async with self._lock:
                     self._nconns -= 1
+                    # If we have given up with a growing attempt, allow a new one.
+                    if growing and self._growing:
+                        self._growing = False
                 self.reconnect_failed()
             else:
                 attempt.update_delay(now)
@@ -441,7 +466,11 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
         await self._add_to_pool(conn)
         if growing:
             async with self._lock:
-                if self._nconns < self._max_size and self._waiting:
+                # Keep on growing if the pool is not full yet, or if there are
+                # clients waiting and the pool can extend.
+                if self._nconns < self._min_size or (
+                    self._nconns < self._max_size and self._waiting
+                ):
                     self._nconns += 1
                     logger.info("growing pool %r to %s", self.name, self._nconns)
                     self.run_task(AddConnection(self, growing=True))
@@ -498,7 +527,7 @@ class AsyncConnectionPool(BasePool[AsyncConnection[Any]]):
 
                 # If we have been asked to wait for pool init, notify the
                 # waiter if the pool is full.
-                if self._pool_full_event and len(self._pool) >= self._nconns:
+                if self._pool_full_event and len(self._pool) >= self._min_size:
                     self._pool_full_event.set()
 
     async def _reset_connection(self, conn: AsyncConnection[Any]) -> None:
